/**
 * Example of how to call the TCB DistributedFailureDetection service.
 * 
 * This server demo is responsible for ending the distributed timing failure detection
 * (and the associated measurement) that is started by client demo. 
 * After ending it, it presents the associated measurement results.
 * 
 * @author Paulo Sousa
 */

#include "stdafx.h"
//include sockets header file
#include <winsock.h>
//include tcb services library header file
#include "..\..\..\tcb_services_library\tcb_services_library.h"

//TODO: replace this values according to your situation
#define MY_IP "192.168.0.2"
#define MY_PORT 40000

int main(int argc, char* argv[]) {
	
	//open_session
	printf("Opening TCB session\n");
	int error = open_tcb_session();
	if (error!=0) {
		printf("Error opening session=%d\n",error);
		getchar();
		return -1;
	}

	//create a TCB socket
	printf("Creating TCB socket\n");
	error = tcb_socket(AF_INET);
	if (error!=0) {
		printf("Error creating TCB socket: %d\n", error);
		getchar();
		return -1;
	}

	//bind TCB socket
	printf("Binding TCB socket\n");
	struct sockaddr_in sa;
	sa.sin_family=AF_INET;
	sa.sin_addr.s_addr= inet_addr(MY_IP);
	sa.sin_port=htons(MY_PORT);
	error=tcb_bind((struct sockaddr*)&sa, sizeof(sa));
	if (error!=0) {
		printf("Error binding TCB socket: %d\n", error);
		getchar();
		return -1;
	}

	//end a distributed fd (and the associated measurement)
	printf("Waiting data\n");
	char buf[50+TCB_DATA_SIZE];
	int buflen = 50+TCB_DATA_SIZE;
	struct sockaddr_in from;
	int fromsize = sizeof(from);
	//used for storing the result of recvfrom. It indicates if the recved data
	//was sent only for measurement or also for timing failure detection.
	tcb_distributed_action_result_t dar;
	
	dar = tcb_recvfrom(buf, &buflen, NULL, (struct sockaddr*)&from, 
						    &fromsize);
	if (dar.type<0) {
		printf("Error receiving data: %d\n", dar.type);
	}
	//if it's a dist. timing failure detection
	else if (dar.type==1) {
		//end failure detection (and the associated measurement)
		tcb_distributed_measurement_info_t mi = 
					end_tcb_distributed_action(dar.fd_info);
		if (mi.delivery_ev<0) {
			printf("Error ending FD: %d\n", mi.delivery_ev);
		}
		else {
			printf("%d bytes received: %s\n", buflen, buf);
			printf("Delay: %d ms, error: %d ms\n", mi.delay, mi.error);
		}
	}
	//if it's only a measurement
	else {
		printf("Error: not expecting msg without FD\n");
	}

	//close_session
	printf("Closing session\n");
	error=close_tcb_session();
	if (error!=0) {
		printf("Error closing session=%d\n",error);
		getchar();
		return -1;
	}
	printf("Close ok!\n");

	//wait for user to press a key
	printf("Press a key to close window...\n");
	getchar();
	return 0;
}
