/**
 * Example of how to define functions/handlers to be used in TCB services.
 * 
 * The services examples use the functions/handlers defined in this example.
 *
 * @author Paulo Sousa
 */

#include "stdafx.h"
#include "dll_example_for_tcb_handler.h"
#include "Winsock.h"

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}


/*TimelyExecution Service */
//function to be timely executed
extern "C" __declspec(dllexport)
void timely_example(void) {
	printf("TimelyExec Handler!\n");
}
//Worst Case Execution Time of timely_example (=1 millisecond)
extern "C" __declspec(dllexport)
int timely_example_wcet(void) {
	return 1;
}


/*LocalFailureDetection Service*/
//function to be executed if a local timing failure occurs
extern "C" __declspec(dllexport)
void localFD_example(void) {
	printf("LocalFailure Handler!\n");
}
//Worst Case Execution Time of localFD_example (=1 millisecond)
extern "C" __declspec(dllexport)
int localFD_example_wcet(void) {
	return 1;
}


/*DistributedFailureDetection Service*/
//function to be executed if a distributed timing failure occurs
extern "C" __declspec(dllexport)
void distFD_example(char from, int seq) {
	//from= tcb id of the msg sender
	//seq= distributed FD sequence number
	printf("DistFailure Handler! from=%d, seq=%d\n", from, seq);
}
//Worst Case Execution Time of distFD_example (=1 millisecond)
extern "C" __declspec(dllexport)
int distFD_example_wcet(void) {
	return 1;
}


/*Remote DistributedFailureDetection Service*/
//function to be executed if a remote distributed timing failure occurs
extern "C" __declspec(dllexport)
void remote_distFD_example(char from, int seq) {
	//from= tcb id of the msg sender
	//seq= distributed FD sequence number
	printf("Remote DistFailure Handler! from=%d, seq=%d\n", from, seq);
}
//Worst Case Execution Time of remote_distFD_example (=1 millisecond)
extern "C" __declspec(dllexport)
int remote_distFD_example_wcet(void) {
	return 1;
}